/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
    Copyright (C) 2020 OpenCFD Ltd.
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::Function1Types::stepFunction

Description
    A step function that is 0 before \c start changing to 1 for the
    \c duration and returning to 0 thereafter.

See also
    Foam::Function1Types::ramp

SourceFiles
    stepFunction.C

\*---------------------------------------------------------------------------*/

#ifndef Function1Types_stepFunction_H
#define Function1Types_stepFunction_H

#include "ramp.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace Function1Types
{

/*---------------------------------------------------------------------------*\
                        Class stepFunction Declaration
\*---------------------------------------------------------------------------*/

class stepFunction
:
    public ramp
{
public:

    //- Runtime type information
    TypeName("step");


    // Constructors

        //- Construct from entry name and dictionary
        stepFunction(const word& entryName, const dictionary& dict);


    //- Destructor
    virtual ~stepFunction() = default;


    // Member Functions

        //- Return value at time t
        virtual inline scalar value(const scalar x) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Function1Types
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "stepFunctionI.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
